/* ----------------------------------------------------------------------*/
/*  frame.c       Teemu Kilpelinen 20.10.2000							 */
/*   												          	         */
/*	This module includes MakeFrame function for frame handling.	    	 */  
/*	Version Number: 0.01											   */
/* ----------------------------------------------------------------------*/

#include <string.h>
#include "frame.h"
#include "s2t_defs.h"


/* --------------------------------------------------------------- */
void MakeFrame(SND_BUFFER signal, int begin, int length, 
	int length_power_of_two, FRAME_ARRAY frame, FLOAT_ARRAY window_coeff, 
	int round_downwards){
/*
	Functioning:	Make frame       Copies samples and multiply them with 
									 window coefficients (Hamming, etc.)
									 from input signal to frame which length 
									 is power of two.
												
	Inputs:			SND_BUFFER signal	    The original signal in a float table.
	                int length              Count of samples which will be 
											copied from signal to frame.
				    int length_power_of_two Frame's length (pover of two). 
				    FRAME_ARRAY frame		Frame (the table where the piece 
											of signal will be copied). 
					FLOAT_ARRAY window_coeff Table of Coefficients (Hamming etc.) 
					int round_downwards     If this is 1 there won't be added 
											zeroes at the begin and end of frame.

	Outputs:		A frame which length is pover of two (it can have nulls at 
					the begin and the end).    
	Return value:	-  
	History:		Teemu Kilpelinen and Tomi Kinnunen	 25.1.2001
	Version:		1.0 (original)

	Reference:		
*/
	int k;
	
	/*j is variable, which in is the index where from the filling begins 
	  (if used not use round) downwards. */
	
	int j = (int)((length_power_of_two - length) / 2.0); 

	memset(frame, 0, sizeof(float) * length_power_of_two);	
	
	if (round_downwards == 1){
		for(k = 0; k < length; k++)
		frame[k] = window_coeff[k] * signal[k + begin];
	}
		
	else { 
		for(k = 0; k < length; k++)
		frame[k+j] = window_coeff[k] * signal[k + begin];
	}
} /* MakeFrame */

