/* --------------------------------------------------------------------*/
/* s2t_defs.c       Teemu Kilpelinen 3.5.2001						   */
/*   												          	       */
/*                                                                     */
/* This module implements functions used to memory handling operations.*/  
/* Version Number: 0.01											       */
/* --------------------------------------------------------------------*/

#include <string.h>
#include "s2t_defs.h"
#include "memctrl.h"


/* --------------------------------------------------------------- */
FLOAT_ARRAY CreateNewFloatArray(int count_of_elements){
/*
  	Functioning:	Allocates memory for float-array and returns pointer 
					to first index.	
	Inputs:			int count_of_elements		

	Outputs:		   
	Return value:	A pointer to new float-array. 
	History:		Teemu Kilpelinen 03.05.2001
	Version:		1.0 (original)
	Reference:		-
*/
	return allocate(sizeof(float) * count_of_elements);
}/* CreateNewFloatArray */


/* --------------------------------------------------------------- */
INT_ARRAY CreateNewIntArray(int count_of_elements){
/*
  	Functioning:	Allocates memory for int-array and returns 
					pointer to first index.	
	Inputs:			int count_of_elements		

	Outputs:		   
	Return value:	A pointer to new int-array. 
	History:		Teemu Kilpelinen 03.05.2001
	Version:		1.0 (original)

	Reference:		-
*/
return allocate(sizeof(int) * count_of_elements);
	 
} /* CreateNewIntArray */


/* --------------------------------------------------------------- */
FEATURE_VECTOR CreateNewFE(int RFCC_ord){
/*
  	Functioning:	
	Inputs:			int RFCC_ord		Count of vector elements.

    Outputs:		   
	Return value:	A pointer to new feature vector 
	History:		Teemu Kilpelinen 27.04.2001
	Version:		1.0 (original)
	Reference:		-
*/

	return memset(allocate(sizeof(feature_element) * RFCC_ord),0,
		sizeof(feature_element) * RFCC_ord);
	 
} /* CreateNewFE */


/* --------------------------------------------------------------- */
FEATURE_TABLE CreateNewFT(int count_of_vectors, int RFCC_ord){
/*
  	Functioning:	Creates new vector element
	Inputs:			int count_of_vectors
					int RFCC_ord		Count of vector elements.
					
	Outputs:		   
	Return value:	A pointer to new feature vector (initialized to zero).
	History:		Teemu Kilpelinen 27.04.2001
	Version:		1.0 (original)
	Reference:		-
*/
	int i;

	FEATURE_TABLE ft;
	
	ft = allocate(sizeof(FEATURE_VECTOR) * count_of_vectors);
	for (i = 0; i < count_of_vectors; i++)
		ft[i] = CreateNewFE(RFCC_ord);
		
	return ft;
} /* CreateNewFT */


/* --------------------------------------------------------------- */
float FeatureElement(float* f, int i){
/*	
	Functioning:	Returns pointer to vector element in index i
	
	Inputs:			float* f	feature vecrtor.
					int i		index.

	Outputs:		   
	Return value:	Pointer of f[i] 
	History:		Teemu Kilpelinen 27.04.2001
	Version:		1.0 (original)

	Reference:		-
*/
	return f[i];
}/* FeatureElement */