/* --------------------------------------------------------------------*/
/*  S2T_DEFS.H       Teemu Kilpelinen 20.10.2000					   */
/*                                                                     */  
/* --------------------------------------------------------------------*/

#ifndef S2T_DEFS_H
#define S2T_DEFS_H

#ifdef __cplusplus
extern "C" {
#endif

#ifndef PI 
#define PI 3.1415926535897932384626433832795
#endif

#define  TRUE     1
#define  FALSE    0


typedef int dBOOLEAN;


typedef unsigned char BYTE;

typedef unsigned char        UINT8;
typedef signed   char        INT8;

typedef unsigned short int   UINT16;
typedef signed   short int   INT16;
typedef unsigned int   		 UINT32;
typedef signed   int   		 INT32;



typedef struct {

	int 
		/* Size of input signal in samples */
		wave_table_size,

		/* Number of channels in the input signal */
		count_of_channels, 
		
		/* Sampling rate of input signal */
		sampling_rate, 
		
		/* Bits of sample of input signal */
		bits_per_sample;

	long   
		/* Count of samples the program managed read to integer_buffer. 
		Must be equal with wave_table_size */
     	count_of_samples;

}SIGNAL_INFORMATION;



typedef struct {

	int		
		/* Length of frame in samples (nearest power of two) */
		length_pow_of_two, 
		
		/* Length of window (frame) shifting in samples */
		win_shift_samples, 
		
		/* Index of frame's starting point */
		begin, 
				
		/* Length of frame (in samples) */
		samples_per_frame;

}FRAME_INFORMATION;
		

typedef struct {


	float*  real_part;
		  
	float*	im_part;

}FFT_FRAME;

/* 	Variables which are loaded from .fac
	parameter struct definitions  */
typedef struct {
		
	int	
		/* window length in ms */
		win_length,
		
		/* window shift in ms */  
		win_shift_ms, 
		
		/* feature-type: MFCC, ..., not implemented yet */
		feat_type, 

		/* number of mel-filters */
		num_mel_filters,

		/* not implemented yet */
		drop_delta, 
		
		/* not implemented yet */
		drop_delta_delta,
		
		/* include 0th coefficient */
		include_0th_coeff,		
		
		/* remove silence, not implemented yet */
		remove_sil, 
		
		/* remove DC-offcet */
		remove_dc_offset, 
		
		/* use nearest smaller power of two */
		use_nearest_small_pow_of_two, 
		
		/* MFCC-order =~ dimension of reusult vector */
		MFCC_ord, 
		
		/* window function type 
		   1 = HAMMING
		   2 = RECTANGULAR
		   3 = BARLETT
		   4 = BLACKMAN */
		win_function_type,
		
		/* lifterfunction type 
		   1 = truncated
		   2 = raised sine
		   3 = half raised sine */
		lifter_type,
		
		/* absolute magnitude */
		absolute_magnitude,
		
		/* Over write existing file*/
		over_write;		
	
		/* high emphasis coefficients */	
	float high_emph_coeff;
}USER_PARAMETERS;

/* array structure definitions */
typedef float float_element;

typedef float_element* FLOAT_ARRAY;


typedef float float_element;

typedef float_element* FRAME_ARRAY;



typedef float float_element;

typedef float_element* SND_BUFFER;
 


typedef int int_element;

typedef int_element* INT_ARRAY;


typedef float feature_element;

typedef feature_element* FEATURE_VECTOR;

typedef FEATURE_VECTOR* FEATURE_TABLE;
/*========================================================================================*/


FLOAT_ARRAY CreateNewFloatArray(int count_of_elements);

INT_ARRAY   CreateNewIntArray(int count_of_elements);

FEATURE_VECTOR CreateNewFE(int RFCC_ord);

FEATURE_TABLE CreateNewFT(int count_of_vectors, int RFCC_ord);

float FeatureElement(float* f, int i);

#ifdef __cplusplus
}
#endif

#endif /* S2T_DEFS.H */

